==> hadoop-3.3.6-src.tar.gz: hadoop-3.3.6-src/hadoop-mapreduce-project/hadoop-mapreduce-client/hadoop-mapreduce-client-shuffle/src/main/java/org/apache/hadoop/mapred/FadvisedChunkedFile.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hadoop.mapred;

import java.io.FileDescriptor;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.apache.hadoop.thirdparty.com.google.common.annotations.VisibleForTesting;
import org.apache.hadoop.io.ReadaheadPool;
import org.apache.hadoop.io.ReadaheadPool.ReadaheadRequest;
import org.apache.hadoop.io.nativeio.NativeIO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.apache.hadoop.io.nativeio.NativeIO.POSIX.POSIX_FADV_DONTNEED;

import org.jboss.netty.handler.stream.ChunkedFile;

public class FadvisedChunkedFile extends ChunkedFile {

  private static final Logger LOG =
      LoggerFactory.getLogger(FadvisedChunkedFile.class);

  private final Object closeLock = new Object();
  private final boolean manageOsCache;
  private final int readaheadLength;
  private final ReadaheadPool readaheadPool;
  private final FileDescriptor fd;
  private final String identifier;

  private volatile ReadaheadRequest readaheadRequest;

  public FadvisedChunkedFile(RandomAccessFile file, long position, long count,
      int chunkSize, boolean manageOsCache, int readaheadLength,
      ReadaheadPool readaheadPool, String identifier) throws IOException {
    super(file, position, count, chunkSize);
    this.manageOsCache = manageOsCache;
    this.readaheadLength = readaheadLength;
    this.readaheadPool = readaheadPool;
    this.fd = file.getFD();
    this.identifier = identifier;
  }

  @VisibleForTesting
  FileDescriptor getFd() {
    return fd;
  }

  @Override
  public Object nextChunk() throws Exception {
    synchronized (closeLock) {
      if (fd.valid()) {
        if (manageOsCache && readaheadPool != null) {
          readaheadRequest = readaheadPool
              .readaheadStream(
                  identifier, fd, getCurrentOffset(), readaheadLength,
                  getEndOffset(), readaheadRequest);
        }
        return super.nextChunk();
      } else {
        return null;
      }
    }
  }

  @Override
  public void close() throws Exception {
    synchronized (closeLock) {
      if (readaheadRequest != null) {
        readaheadRequest.cancel();
        readaheadRequest = null;
      }
      if (fd.valid() &&
          manageOsCache && getEndOffset() - getStartOffset() > 0) {
        try {
          NativeIO.POSIX.getCacheManipulator().posixFadviseIfPossible(
              identifier,
              fd,
              getStartOffset(), getEndOffset() - getStartOffset(),
              POSIX_FADV_DONTNEED);
        } catch (Throwable t) {
          LOG.warn("Failed to manage OS cache for " + identifier +
              " fd " + fd.toString(), t);
        }
      }
      // fd becomes invalid upon closing
      super.close();
    }
  }
}

==> hadoop-3.4.1-src.tar.gz: hadoop-3.4.1-src/hadoop-mapreduce-project/hadoop-mapreduce-client/hadoop-mapreduce-client-shuffle/src/main/java/org/apache/hadoop/mapred/FadvisedChunkedFile.java <==
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.hadoop.mapred;

import java.io.FileDescriptor;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.apache.hadoop.classification.VisibleForTesting;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;
import io.netty.handler.stream.ChunkedFile;
import org.apache.hadoop.io.ReadaheadPool;
import org.apache.hadoop.io.ReadaheadPool.ReadaheadRequest;
import org.apache.hadoop.io.nativeio.NativeIO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.apache.hadoop.io.nativeio.NativeIO.POSIX.POSIX_FADV_DONTNEED;

public class FadvisedChunkedFile extends ChunkedFile {

  private static final Logger LOG =
      LoggerFactory.getLogger(FadvisedChunkedFile.class);

  private final Object closeLock = new Object();
  private final boolean manageOsCache;
  private final int readaheadLength;
  private final ReadaheadPool readaheadPool;
  private final FileDescriptor fd;
  private final String identifier;

  private volatile ReadaheadRequest readaheadRequest;

  public FadvisedChunkedFile(RandomAccessFile file, long position, long count,
      int chunkSize, boolean manageOsCache, int readaheadLength,
      ReadaheadPool readaheadPool, String identifier) throws IOException {
    super(file, position, count, chunkSize);
    this.manageOsCache = manageOsCache;
    this.readaheadLength = readaheadLength;
    this.readaheadPool = readaheadPool;
    this.fd = file.getFD();
    this.identifier = identifier;
  }

  @VisibleForTesting
  FileDescriptor getFd() {
    return fd;
  }

  @Override
  public ByteBuf readChunk(ByteBufAllocator allocator) throws Exception {
    synchronized (closeLock) {
      if (fd.valid()) {
        if (manageOsCache && readaheadPool != null) {
          readaheadRequest = readaheadPool
              .readaheadStream(
                  identifier, fd, currentOffset(), readaheadLength,
                  endOffset(), readaheadRequest);
        }
        return super.readChunk(allocator);
      } else {
        return null;
      }
    }
  }

  @Override
  public void close() throws Exception {
    synchronized (closeLock) {
      if (readaheadRequest != null) {
        readaheadRequest.cancel();
        readaheadRequest = null;
      }
      if (fd.valid() &&
          manageOsCache && endOffset() - startOffset() > 0) {
        try {
          NativeIO.POSIX.getCacheManipulator().posixFadviseIfPossible(
              identifier,
              fd,
              startOffset(), endOffset() - startOffset(),
              POSIX_FADV_DONTNEED);
        } catch (Throwable t) {
          LOG.warn("Failed to manage OS cache for " + identifier +
              " fd " + fd.toString(), t);
        }
      }
      // fd becomes invalid upon closing
      super.close();
    }
  }
}
